#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['mixupy']

package_data = \
{'': ['*'], 'mixupy': ['.backup/*', '.backup/auto/*']}

setup(name='mixupy',
      version='0.1.2',
      description='Data augmentation with the mixup method',
      author='makeyourownmaker',
      author_email='makeyourownmaker@gmx.com',
      url='https://github.com/makeyourownmaker/mixupy',
      packages=packages,
      package_data=package_data,
      python_requires='>=3.7',
     )
